class DZMProgressbar{
    constructor(selector = false) {

        this.markElements = [];
        this.marks = [];
        this.goalWrapper = false
        this.barColor = '#A9A29D'
        this.progressColor = '#BFD037'
        this.fillColor = '#ffffff'
        this.showInfoText = true
        this.unitBefore = false

        this.offset = 0;
        this.state = 0;
        this.goal = 0;

        this.nextGoal = 0;
        this.prevGoal = 0;

        this.wrapper = false;
        this.barWrapper = false;
        this.progressbar = false;
        this.mode = 'all'; // all|single|next
        this.showSpecialMark = true;
        this.markPoints = {
            min:false,
            prev:false,
            next:false,
            special:false,
            max:false,
        }
        this.infoTextWrapper = false;

        this.selectors = {
            wrapper:selector === false ? '[data-dzm-pb-wrapper]' : selector,
            barWrapper:'.dzm-pb',
            goalWrapper:"[data-dzm-pb-goal]",
            marker:'[data-dzm-pb-marker]',
            markerValue:'data-dzm-pb-marker',
            markerType:'data-dzm-pb-marker-type',
            progressbar:'[data-dzm-pb-progress]',
            progressbarValue:'data-dzm-pb-progress',
            localStorageProgressState:'dzmProgressState'+(selector === false ? '.dzm-pb' : selector),
            localStorageLastGoal:'dzmLastGoal'+(selector === false ? '.dzm-pb' : selector),
            infoTextWrapper:'[data-dzm-pb-info-text]'
        }

        this.settings={
            duration:1,
            fps:60
        }

        this.strings = {
            outOf:'von',
            unit:'',
            stateTextBefore:'',
            stateTextAfter:'',
        }
    }

    init(options){
        console.warn('INIT PROGRESSBAR',this.selectors.wrapper)

        // GET COMPONENTS
        this.wrapper = document.querySelector(this.selectors.wrapper)
        if(!this.wrapper){
            console.log('no wrapper', this.selectors.wrapper)
            return
        }

        //dzmDispatcher.observeIntersection(this.wrapper,dzmDispatcher.events.progressBar.intersecting,'off',true)

        /*dzmDispatcher.on(dzmDispatcher.events.progressBar.intersecting,()=>{
            console.log('fuck that works')
        })*/

        this.barWrapper = this.wrapper.querySelector(this.selectors.barWrapper)
        // SET WIDTH FOR INITIAL OPEN LOADING EFFECT
        this.barWrapper.style.width = '100%';
        // GET PROGRESSBAR COMPONENT AND MARKS
        this.progressbar = this.wrapper.querySelector(this.selectors.progressbar)
        this.markElements = this.wrapper.querySelectorAll(this.selectors.marker)
        //console.log('MARK ELEMENTS', this.markElements)
        this.goalWrapper = this.wrapper.querySelector(this.selectors.goalWrapper)



        // GET OPTIONS
        this.mode = options.mode || this.mode;
        this.barColor = options.barColor || this.barColor;
        this.progressColor = options.progressColor || this.progressColor;
        this.fillColor = options.fillColor || this.fillColor;
        this.showInfoText = 'showInfoText' in options ? options.showInfoText : this.showInfoText;
        this.showSpecialMark = 'showSpecialMark' in options ? options.showSpecialMark : this.showSpecialMark;

        if(this.showInfoText){
            this.infoTextWrapper = this.wrapper.querySelector(this.selectors.infoTextWrapper)
        }

        this.strings = {
            outOf:options.strings.outOf||this.strings.outOf,
            unit:options.strings.unit||this.strings.unit,
            stateTextBefore:options.strings.stateTextBefore||this.strings.stateTextBefore,
            stateTextAfter:options.strings.stateTextAfter||this.strings.stateTextAfter,
        };
        this.unitBefore = options.strings.unitBefore||this.unitBefore,

        // GET CURRENT TARGET VALUE BY ATTRIBUTE
        //this.currentTarget = parseFloat(this.progressbar.getAttribute(this.selectors.progressbarValue));
        this.goal = parseFloat(this.progressbar.getAttribute(this.selectors.progressbarValue));
        //console.error('GOAL', this.goal)
        ////console.warn('currentTarget', this.currentTarget)
        this.state = this.goal;
        if((this.state = localStorage.getItem(this.selectors.localStorageLastGoal)) === null){
            this.state =0
            localStorage.setItem(this.selectors.localStorageLastGoal, this.state);


        }else{
            // DEACTIVATE ANIMATION, SO IT IS NOT ANIMATING ALL THE TIME
            this.barWrapper.classList.toggle('dzm-pb--no-animation',true)
        }
        this.state = parseFloat(this.state)

        //console.error('STATE', this.state)
        // console.warn('LAST TARGET', this.lastTarget)



        this.markElements.forEach(marker=>{
            //console.log(marker.dataset)
            let markerValue = parseFloat(marker.getAttribute(this.selectors.markerValue));
            let markerType = marker.getAttribute(this.selectors.markerType);
            let newMark = {
                value:markerValue,
                type: markerType,
                mark:marker,
                description:marker.dataset.dzmPbMarkerDescription
            }
            this.marks.push(newMark)


        })


        this.marks.sort((a, b) => a.value - b.value);

        this.setupMarkPoints();


        //console.warn('markElements', this.markElements)
        //console.warn('marks', this.marks)
        //console.warn('markPoints', this.markPoints)

        this.setMarks()


        //console.warn('MARKS SET UP')
        let statePercent = (this.state - this.prevGoal) * 100 / (this.nextGoal - this.prevGoal === 0 ? 1 : this.nextGoal - this.prevGoal)
        //console.error('state', this.state)
        //console.error('this.prevGoal', this.prevGoal)
        //console.error('this.nextGoal', this.nextGoal)

        //console.error('STATE PERCENT', statePercent)
        this.progressbar.style.width =statePercent + '%';

        // VIEW THE PROGRESS BAR
        this.barWrapper.style.opacity = '1';
        // ACTIVATE ANIMATIONS
        setTimeout(()=>{
            this.barWrapper.classList.toggle('dzm-pb--no-animation',false)
        },1)

        window.progressBar = this
        this.setProgress( this.goal)
    }
    setGoalString(){
        //console.warn('SETGOALSTRING',this.marks)
        if(!this.goalWrapper && !this.infoTextWrapper)
            return;
        let set = false;
        this.marks.forEach(mark=>{
            if((mark.value > this.goal || mark.isMax) && !set){
                if(this.goalWrapper){
                    this.goalWrapper.innerText = Math.min(this.goal,this.marks[this.markPoints.max].value).toFixed(2) + ' '+this.strings.outOf+' ' +mark.value.toFixed(2)
                }
                if(this.showInfoText && mark.value-this.goal > 0){
                    let infoText = '';
                    infoText = this.strings.stateTextBefore+ ' ';
                    if(this.unitBefore){
                        infoText += this.strings.unit + ' ' + (mark.value-this.goal).toFixed(2) + ' ';
                    }else{
                        infoText += (mark.value-this.goal).toFixed(2) + ' ' + this.strings.unit + ' ';
                    }
                    infoText += mark.description + ' ' + this.strings.stateTextAfter;

                    this.infoTextWrapper.innerText = infoText;
                }else{
                    this.infoTextWrapper.innerText = '';

                }
                set = true
            }

        })
    }

    setProgress( target=0){

        //console.warn('SET PROGRESS')
        //console.error('state', this.state)
        //console.error('goal', this.goal)
        //console.error('target', target)
        //console.error('nextGoal' ,this.nextGoal)
        //console.error('prevGoal' ,this.prevGoal)

        // CALC THE DIFF PERCENTAGE OF TARGET FROM SAVEDPROGRESS, BUT MAX 100%

        let statePercent = (this.state - this.prevGoal) * 100 / (this.nextGoal - this.prevGoal === 0 ? 1 : this.nextGoal - this.prevGoal)
        //console.error('START FROM X PERCENT' ,statePercent)

        let targetPercent = ((Math.min(target,this.nextGoal) - this.prevGoal) * 100 / ((this.nextGoal - this.prevGoal === 0 ? 1 : this.nextGoal - this.prevGoal))) - statePercent;
        //console.error('GOAL FOR THIS ROUND IS AT ', targetPercent)




        let time = 0;
        let interval = setInterval(()=>{
            //position += increment;
            time += 1 / this.settings.fps;
            let progress = this.easeInOutQuad(time * 100 / this.settings.duration, time, 0, targetPercent, this.settings.duration);
            let currentProgress = statePercent + progress;

            this.progressbar.style.width =currentProgress + '%';
            this.marks.forEach(mark=>{
                let markPercentValue = (mark.value-this.prevGoal) * 100 / (this.nextGoal-this.prevGoal);

                this.toggleDone(mark.mark,markPercentValue <= currentProgress)
            })

            if (currentProgress === (targetPercent + statePercent)) {
                // hier erreicht man den maximal möglichen Progress. Bei All wäre das das letzte Ziel oder die maximal erreichten Prozent, bei single und next wäre das maximal das nächste Ziel
                // oder die erreichten Prozent
                clearInterval(interval);
                this.state = Math.min(target,this.nextGoal)
                localStorage.setItem(this.selectors.localStorageLastGoal, this.state);
                //console.error('markpoints.next',this.markPoints.next)
                //console.error('markPoints.max',this.markPoints.max)
                //console.error('nextGoal',this.nextGoal)
                //console.error('markPoints.prev',this.markPoints.prev)
                //console.error('target',target)


                if(this.mode !== 'all' && (
                    (this.markPoints.next !== false && this.markPoints.next !== this.markPoints.max  )
                    || (this.markPoints.prev !== false && target <= this.marks[this.markPoints.prev].value))){
                    //console.log('CHECK')
                    this.checkNextGoales(currentProgress, target)
                }
            }
        }, 1000 / this.settings.fps);


    }

    toggleDone(mark,on = true){
        mark.classList.toggle('done', on)
        if(on){
            mark.firstElementChild.style.backgroundColor = this.progressColor;
        }else{
            mark.firstElementChild.style.backgroundColor = this.barColor;
        }
    }

    checkNextGoales(currentProgress,target){
        //console.warn('CHECK NEXT GOALS')
        //console.error('CURRENT PROGRESS', currentProgress)
        //console.error('TARGET',  target)
        //console.error('MAX TARGET',  this.maxTarget)
        // IF TARGET IS BIGGER THAN CURRENT MAX TARGET

        let lastNextGoal = this.nextGoal;
        let lastPrevGoal = this.prevGoal;


        this.setupMarkPoints()
        this.setMarks()

        if(lastNextGoal === this.nextGoal && lastPrevGoal === this.prevGoal)
            return;
        let statePercent = (this.state - this.prevGoal) * 100 / (this.nextGoal - this.prevGoal === 0 ? 1 : this.nextGoal - this.prevGoal)
        this.progressbar.style.width =statePercent + '%';



        this.setProgress(target)



    }

    setupMarkPoints(){
        this.marks.forEach(mark=>{
            mark.isMin = false;
            mark.isMax = false;
            mark.isPrev = false;
            mark.isSpecial = false;
            mark.isNext = false;
            this.markPoints.min = false;
            this.markPoints.max = false;
            this.markPoints.prev = false;
            this.markPoints.next = false;
            this.markPoints.special = false;
        })
        this.marks.forEach((mark,i)=>{
            if(i===0){
                this.markPoints.min = i;
                mark.isMin = true;
            }
            if(i === this.marks.length - 1){
                this.markPoints.max = i;
                mark.isMax = true;
            }

            if(mark.type === 'special'){
                this.markPoints.special = i;
                mark.isSpecial = true;
            }

            if(mark.value <= this.state && !mark.isMax){
                this.markPoints.prev = i;
                mark.isPrev = true;
                if(i>0){
                    this.marks[i-1].isPrev = false
                }
            }

            if(mark.value > this.state && this.markPoints.next === false){
                this.markPoints.next = i;
                mark.isNext = true;
            }
        })

        if(this.mode === 'all'){
            this.nextGoal = this.marks[this.markPoints.max].value
            this.prevGoal = 0;
        }else if(this.mode === 'next'){
            if(this.markPoints.next !== false){
                if(this.showSpecialMark && ((this.markPoints.special !== false && this.marks[this.markPoints.special].value > this.marks[this.markPoints.next].value)
                    /*|| (this.markPoints.special === false && this.marks[this.markPoints.special].value > this.marks[this.markPoints.next].value)*/
                )){
                    this.nextGoal = this.marks[this.markPoints.special].value;
                }else{
                    this.nextGoal = this.marks[this.markPoints.next].value;
                }
            }else{
                this.nextGoal = this.marks[this.markPoints.max].value;
            }

            this.prevGoal = 0;
        }else if(this.mode ==='single'){
            if(this.markPoints.next !== false){
                if(this.showSpecialMark && this.markPoints.special !== false && this.marks[this.markPoints.special].value > this.marks[this.markPoints.next].value){
                    this.nextGoal = this.marks[this.markPoints.special].value;
                }else{
                    this.nextGoal = this.marks[this.markPoints.next].value;
                }
            }else{
                this.nextGoal = this.marks[this.markPoints.max].value;
            }
            if(this.markPoints.prev !== false){
                this.prevGoal = this.marks[this.markPoints.prev].value;
            }else{
                this.prevGoal = 0;
            }

        }

        //console.warn('SETUP MARK POINTS')
        //console.error('this.marks', this.marks)
        //console.error('this.markPoints', this.markPoints)
        //console.error('this.nextGoal', this.nextGoal)
        //console.error('this.prevGoal', this.prevGoal)

    }

    setMarks(){
        //console.warn('SET MARKS',this.marks)
        //console.log('prevGoal',this.prevGoal)
        this.marks.forEach((mark,i)=>{
            let position = 0;
            mark.mark.style.zIndex = 'unset';
            switch (this.mode){
                case 'all':
                    position = mark.value * 100 / this.nextGoal
                    mark.mark.style.left = position + '%';
                    this.toggleDone(mark.mark,mark.value <= this.state)
                    break;
                case 'next':
                    position = mark.value * 100 / this.nextGoal

                    console.log(mark);
                    mark.mark.style.left = position + '%';
                    this.toggleDone(mark.mark,mark.value <= this.state)
                    if(mark.value > this.state){
                        //console.warn('MARK VALUE IS GREATER THAN STATE', mark.value, this.state)
                        if (mark.isNext || (this.showSpecialMark && mark.isSpecial) || (this.showSpecialMark && mark.value <= this.marks[this.markPoints.special].value)) {
                            //console.warn('YOU SHALL NOT BE VISIBLE')


                        }else{
                            mark.mark.style.zIndex = '-1';
                        }
                    }

                    break;
                case 'single':
                    if(mark.isMax){
                        position = 100;
                    }else{

                    position = Math.max(0,(mark.value - this.prevGoal) * 100 / (this.nextGoal - this.prevGoal))
                    }

                    mark.mark.style.left = position + '%';
                    this.toggleDone(mark.mark,mark.value <= this.state)
                    if(mark.value > this.state){
                        //console.warn('MARK VALUE IS GREATER THAN STATE', mark.value, this.state)
                        if (mark.isNext || (this.showSpecialMark && mark.isSpecial) || (this.showSpecialMark && mark.value <= this.marks[this.markPoints.special].value)) {
                            //console.warn('YOU SHALL NOT BE VISIBLE')


                        }else{
                            mark.mark.style.zIndex = '-1';
                        }
                    }


            }
            //console.error('mark', mark)
            //console.error('type', mark.type, mark.value)
            //console.error('POSITION', position)
            //console.error('DONE',mark.value <= this.state )
            //console.error('z-Index',mark.mark.style.zIndex)
        })

        this.setGoalString()

    }

    easeInOutQuad(x, t, b, c, d) {
        if ((t /= d / 2) < 1) {
            return c / 2 * Math.pow(t, 4) + b;
        } else {
            let adjustedT = t - 2;
            return -c / 2 * (Math.pow(adjustedT, 4) - 2) + b;
        }
    }

    testSteps(time = 1500){
        console.error('START TEST testSteps ',time)
            console.warn('test 40')
            this.setProgress(40);
        setTimeout(()=>{
                console.warn('test 50')
                this.setProgress(50);
            setTimeout(()=>{
                    console.warn('test 60')
                    this.setProgress(60);
                setTimeout(()=>{
                        console.warn('test 100')
                        this.setProgress(100);
                    setTimeout(()=>{
                            console.warn('test 110')
                            this.setProgress(110);
                        setTimeout(()=>{
                                console.warn('test 150')
                                this.setProgress(150);
                            setTimeout(()=>{
                                    console.warn('test 160')
                                    this.setProgress(160);
                                setTimeout(()=>{
                                        console.warn('test 200')
                                        this.setProgress(200);
                                    setTimeout(()=>{
                                        console.warn('test 210')
                                        this.setProgress(210);
                                        console.warn('TEST DONE')

                                    },time)
                                },time)
                            },time)
                        },time)
                    },time)
                },time)
            },time)
        },time)


    }

    testStepOver(time = 2000){
        console.error('START TESTS testStepOver ',time)
        console.warn('test 60')
        this.setProgress(60);
        setTimeout(()=>{
            console.warn('test 110')
            this.setProgress(110);
            setTimeout(()=>{
                console.warn('test 160')
                this.setProgress(160);
                setTimeout(()=>{
                    console.warn('test 210')
                    this.setProgress(210);
                    console.warn('TEST DONE')

                },time)
            },time)
        },time)
    }

    testStepAll(){
        console.error('START TESTS testStepAll ')
        console.warn('test 210')
        this.setProgress(210);
    }


    testStepsBackwards(time = 1500){
        console.error('START TESTS testStepsBackwards ',time)
        console.warn('test 40')
        this.setProgress(200);
        setTimeout(()=>{
            console.warn('test 50')
            this.setProgress(190);
            setTimeout(()=>{
                console.warn('test 60')
                this.setProgress(150);
                setTimeout(()=>{
                    console.warn('test 100')
                    this.setProgress(140);
                    setTimeout(()=>{
                        console.warn('test 110')
                        this.setProgress(100);
                        setTimeout(()=>{
                            console.warn('test 150')
                            this.setProgress(90);
                            setTimeout(()=>{
                                console.warn('test 160')
                                this.setProgress(50);
                                setTimeout(()=>{
                                    console.warn('test 200')
                                    this.setProgress(40);
                                    setTimeout(()=>{
                                        console.warn('test 210')
                                        this.setProgress(0);
                                        console.warn('TEST DONE')

                                    },time)
                                },time)
                            },time)
                        },time)
                    },time)
                },time)
            },time)
        },time)


    }

    testStepOverBackwards(time = 2000){
        console.error('START TESTS testSTepOverBackwards',time)
        console.warn('test 60')
        this.setProgress(190);
        setTimeout(()=>{
            console.warn('test 110')
            this.setProgress(140);
            setTimeout(()=>{
                console.warn('test 160')
                this.setProgress(90);
                setTimeout(()=>{
                    console.warn('test 210')
                    this.setProgress(40);
                    setTimeout(()=>{
                        console.warn('test 210')
                        this.setProgress(0);
                        console.warn('TEST DONE')

                    },time)
                },time)
            },time)
        },time)
    }


    testStepAllBackwards(){
        console.error('START TEST testSteppAllBackwards')
        console.warn('test 210')
        this.setProgress(0);
    }


    testItAll(){
        this.testSteps()
        setTimeout(()=>{
            this.testStepsBackwards()
            setTimeout(()=>{
                this.testStepOver()
                setTimeout(()=>{
                    this.testStepOverBackwards()
                    setTimeout(()=>{
                        this.testStepAll()
                        setTimeout(()=>{
                            this.testStepAllBackwards()
                        },2*2000)
                    },5*2000)
                },5*2000)
            },10*1500)
        },10*1500)




    }

}